#!/bin/bash
#
# VERSION=2
# CHANGES="Adjustments to DHCP-provisioning"

# Constants #

beroconf="/usr/fallback/beroconf"
FWIMGDIR=/tmp/images

# main #
if [ ! -d "${FWIMGDIR}" ]; then
	mkdir -p ${FWIMGDIR}
fi

prov_type=$(${beroconf} get root dhcp-prov-type | grep -v failed)
if [ -z "${prov_type}" ]; then
	prov_type=$(${beroconf} get root provisioning-option | grep -v failed)
fi


cd /usr/conf

function get_file {

	case "${prov_type}" in
		http)
			http_host=$(${beroconf} get root http-host | grep -v failed)
			http_path=$(${beroconf} get root http-url | grep -v failed)

			if [ -z "${http_host}" ] || [ -z "${http_path}" ]; then
				return 1
			fi

			echo "  == http fetching file ${http_path}/${1}"

			eval $(cat /proc/cmdline)
			SERIAL=${carduuid}
			MAC=$(/sbin/ifconfig eth0 |/bin/grep HWaddr | awk '{ print $5 }')

			http_path=$(echo ${http_path} | sed "s/^\///")
			http_path=$(echo ${http_path} | sed "s/\/$//")
			http_path=$(echo ${http_path} | sed "s/{mac}/${MAC}/")
			http_path=$(echo ${http_path} | sed "s/{serial}/${SERIAL}/")
			http_path1=$(echo ${http_path} | sed "s/{file}/${1}/")

#			echo $http_path
#			echo $http_path1

			if [ "${http_path}" = "${http_path1}" ] ; then
#				echo "/usr/bin/wget http://$http_host/$http_path/$1"
				/usr/bin/wget http://$http_host/$http_path/$1
			else
#				echo "/usr/bin/wget http://$http_host/$http_path1"
				/usr/bin/wget http://$http_host/$http_path1
			fi

#			prepare to transmit user-agent when busybox wget supports this option
#			/usr/bin/wget --user-agent '<berofix:'$Hwaddress'>' http://$http_host/$http_path/
			;;
		tftp)
			host=$(${beroconf} get root dhcp-tftp-host | grep -v failed)
			path=$(${beroconf} get root dhcp-tftp-path | grep -v failed)

			if [ -z "${host}" ]; then
				host=$(${beroconf} get root tftp-host | grep -v failed)
				path=$(${beroconf} get root tftp-url | grep -v failed)
			fi

			if [ -n "${host}" ]; then
				echo "  == tftp fetching file ${path}/${1}"
				/usr/local/bin/tftp ${host} > /dev/null <<EOF
timeout 1
get ${path}/${1}
EOF
			fi
			;;
	esac
}

function check_file {

	if [ ! -s /usr/conf/${1} ]; then
		cp -f /usr/conf/backup/${1} /usr/conf/
		echo "missing file " ${1} " has been replaced"
	fi
}

function reload_settings {

        echo "reloading settings"
        STR=$(echo 'rd'|nc localhost 54322 -q1)
        sleep 1
        STR=$(echo 'rs'|nc localhost 54322 -q1)
}

function update_config {

	ret=1

        if [ -s conf-update.conf ]; then
                . conf-update.conf
                if [ "$CONF_DOWNLOAD" = "yes" ] || [ "$CONF_DOWNLOAD" = "1" ]; then
                        #echo "  = $prov_type update: checking for files on $host:/$url"
                        mkdir -p backup
                        cp -f isgw.* backup/
                        cp -f *.conf backup/
                        rm -rf isgw.conf isgw.sip isgw.dialplan isgw.isdn isgw.gsm
                        rm -rf hardware.conf isgw.analog misc.conf filter.conf isgw.tones isgw.causes
                        sync

			file_list="hardware.conf isgw.conf isgw.dialplan isgw.analog isgw.isdn isgw.gsm isgw.sip isgw.causes isgw.tones misc.conf filter.conf"
			for file in $(echo ${file_list}); do
				get_file ${file}
			done

                        sync
                        sleep 1

			if [ ! -s hardware.conf ] || [ ! -s isgw.conf ] || [ ! -s isgw.dialplan ] || [ ! -s isgw.sip ]; then
				rm -f /usr/conf/conf-update.conf
				sync
				echo "config_update:${prov_type}:error:Download_Config_Files_Failed"
				exit 0
			fi

			file_list="isgw.isdn isgw.analog isgw.gsm misc.conf isgw.causes isgw.tones filter.conf"
			for file in $(echo ${file_list}); do
				check_file ${file}
			done

			/usr/bin/env -i bash -c "/usr/local/php/ini_to_db.php"

			if [ "${?}" -eq 0 ] ; then
				sync
				echo "config_update:${prov_type}:success"
			else
				echo "config_update:${prov_type}:error:Check_Config_Files"
			fi
        	else
                        echo "config_update:${prov_type}:failed:No_Config_Download_Selected"
			ret=0
                fi
                sync
	else
        	echo "config_update:${prov_type}:failed:Downloading_conf-update.conf"
		ret=1
	fi

	rm -f /usr/conf/conf-update.conf
	sync

	return ${ret}
}

function upgrade_firmware {

	if [ -s firmware-update.conf ] ; then

		. firmware-update.conf

		ret=1

		# check if installed version is >= provisioned
		if [ -f "/usr/local/FILENAME" ]; then
			INST_FILE=$(cat /usr/local/FILENAME)
			inst_vers=$(expr match "${INST_FILE}" ".*appfs-\(.*\)\.tar\.gz")
			prov_vers=$(expr match "${FIRMWARE_FILE}" ".*appfs-\(.*\)\.tar\.gz")

			if [ ! -z "${prov_vers}" ]; then
				/usr/sbin/comp-app-vers.sh ${inst_vers} ${prov_vers}
				if [ ${?} -eq 0 ]; then
					echo "firmware_update:${prov_type}:success:Already_InstalledOrNewer_'${FIRMWARE_FILE}'"
					return 0
				fi
			fi
		fi

		if [ "$FIRMWARE_DOWNLOAD" = "yes" ] || [ "$FIRMWARE_DOWNLOAD" = "1" ] ; then
			cd ${FWIMGDIR}
			rm -rf *
			get_file $FIRMWARE_FILE

			if ! [ -s $FWIMGDIR/$FIRMWARE_FILE ] ; then
				echo "firmware_update:$prov_type:failed:Invalid_Firmware_File_'$FIRMWARE_FILE'"
				ret=1
			else
				if [ "$FIRMWARE_INSTALL" = "yes" ] || [ "$FIRMWARE_INSTALL" = "1" ] ; then
					${beroconf} $FIRMWARE_FILE
					if [ "${?}" != "0" ] ; then
						echo "firmware_update:$prov_type:error:Firmware_Archive_Integrity"
						ret=1
					else
						${beroconf} set root app-image $FIRMWARE_FILE
						echo "firmware_update:$prov_type:success:Installed_'$FIRMWARE_FILE'"
						ret=0
					fi
				else
					echo "firmware_update:$prov_type:failed:Install=No_Selected"
					ret=0
				fi
			fi
		else
			echo "firmware_update:$prov_type:Download=No_Selected"
			ret=0
		fi
	else
		echo "firmware_update:$prov_type:failed:Missing_File_firmware-update.conf"
		ret=1
	fi

	rm -f /usr/conf/firmware-update.conf
	sync

	return ${ret}
}

function restoreoldconfig {
	echo "Restore old Config"
	cp -f backup/isgw.* .
	cp -f backup/*.conf .
	/usr/bin/env -i bash -c "/usr/local/php/ini_to_db.php"
	sync
	sleep 1
}


### MAIN ###

# first refresh config files from Database
wget -O - http://localhost/app/berogui/includes/create_files.php >> /dev/null

case "${1}" in
	config)
		get_file conf-update.conf
		update_config
		exit ${?}
		;;
	firmware)
		get_file firmware-update.conf
		upgrade_firmware
		ret=${?}

		if [ -f /tmp/reboot_after_update ]; then
			/sbin/reboot
		fi

		exit ${ret}
		;;
	restore)
		if [ -s backup/hardware.conf ] || [ -s backup/isgw.conf ] || [ -s backup/isgw.dialplan ] || [ -s backup/isgw.sip ]; then
			restoreoldconfig
			echo "restoreoldconfig:success"
		else
			echo "restoreoldconfig:failed"
		fi
		;;
esac

